"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.postEvaluateRoute = void 0;
var _securitysolutionEsUtils = require("@kbn/securitysolution-es-utils");
var _std = require("@kbn/std");
var _langsmith = require("langsmith");
var _evaluation = require("langsmith/evaluation");
var _uuid = require("uuid");
var _server = require("@kbn/data-plugin/server");
var _elasticAssistantCommon = require("@kbn/elastic-assistant-common");
var _common = require("@kbn/elastic-assistant-common/impl/schemas/common");
var _server2 = require("@kbn/langchain/server");
var _agents = require("langchain/agents");
var _chains = require("langchain/chains");
var _build_response = require("../../lib/build_response");
var _helpers = require("../helpers");
var _constants = require("../knowledge_base/constants");
var _utils = require("./utils");
var _helpers2 = require("../../ai_assistant_data_clients/anonymization_fields/helpers");
var _elasticsearch_store = require("../../lib/langchain/elasticsearch_store/elasticsearch_store");
var _graph = require("../../lib/langchain/graphs/default_assistant_graph/graph");
var _prompts = require("../../lib/langchain/graphs/default_assistant_graph/prompts");
var _utils2 = require("../utils");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const DEFAULT_SIZE = 20;
const ROUTE_HANDLER_TIMEOUT = 10 * 60 * 1000; // 10 * 60 seconds = 10 minutes

const postEvaluateRoute = (router, getElser) => {
  router.versioned.post({
    access: _elasticAssistantCommon.INTERNAL_API_ACCESS,
    path: _elasticAssistantCommon.ELASTIC_AI_ASSISTANT_EVALUATE_URL,
    options: {
      tags: ['access:elasticAssistant'],
      timeout: {
        idleSocket: ROUTE_HANDLER_TIMEOUT
      }
    }
  }).addVersion({
    version: _elasticAssistantCommon.API_VERSIONS.internal.v1,
    validate: {
      request: {
        body: (0, _common.buildRouteValidationWithZod)(_elasticAssistantCommon.PostEvaluateBody)
      },
      response: {
        200: {
          body: {
            custom: (0, _common.buildRouteValidationWithZod)(_elasticAssistantCommon.PostEvaluateResponse)
          }
        }
      }
    }
  }, async (context, request, response) => {
    const ctx = await context.resolve(['core', 'elasticAssistant', 'licensing']);
    const assistantContext = ctx.elasticAssistant;
    const actions = ctx.elasticAssistant.actions;
    const logger = assistantContext.logger.get('evaluate');
    const telemetry = assistantContext.telemetry;
    const abortSignal = (0, _server.getRequestAbortedSignal)(request.events.aborted$);

    // Perform license, authenticated user and evaluation FF checks
    const checkResponse = (0, _helpers.performChecks)({
      authenticatedUser: true,
      capability: 'assistantModelEvaluation',
      context: ctx,
      license: true,
      request,
      response
    });
    if (checkResponse) {
      return checkResponse;
    }
    try {
      var _await$assistantConte, _await$assistantConte2, _await$assistantConte3, _kbDataClient$indexTe, _kbDataClient$indexTe2;
      const evaluationId = (0, _uuid.v4)();
      const {
        alertsIndexPattern,
        datasetName,
        graphs: graphNames,
        langSmithApiKey,
        connectorIds,
        size,
        replacements,
        runName = evaluationId
      } = request.body;
      const dataset = await (0, _utils.fetchLangSmithDataset)(datasetName, logger, langSmithApiKey);
      if (dataset.length === 0) {
        return response.badRequest({
          body: {
            message: `No LangSmith dataset found for name: ${datasetName}`
          }
        });
      }
      logger.info('postEvaluateRoute:');
      logger.info(`request.query:\n${JSON.stringify(request.query, null, 2)}`);
      logger.info(`request.body:\n${JSON.stringify(request.body, null, 2)}`);
      logger.info(`Evaluation ID: ${evaluationId}`);
      const totalExecutions = connectorIds.length * graphNames.length * dataset.length;
      logger.info('Creating graphs:');
      logger.info(`\tconnectors/models: ${connectorIds.length}`);
      logger.info(`\tgraphs: ${graphNames.length}`);
      logger.info(`\tdataset: ${dataset.length}`);
      logger.warn(`\ttotal graph executions: ${totalExecutions} `);
      if (totalExecutions > 50) {
        logger.warn(`Total baseline graph executions >= 50! This may take a while, and cost some money...`);
      }

      // Setup graph params
      // Get a scoped esClient for esStore + writing results to the output index
      const esClient = ctx.core.elasticsearch.client.asCurrentUser;

      // Default ELSER model
      const elserId = await getElser();

      // Data clients
      const anonymizationFieldsDataClient = (_await$assistantConte = await assistantContext.getAIAssistantAnonymizationFieldsDataClient()) !== null && _await$assistantConte !== void 0 ? _await$assistantConte : undefined;
      const conversationsDataClient = (_await$assistantConte2 = await assistantContext.getAIAssistantConversationsDataClient()) !== null && _await$assistantConte2 !== void 0 ? _await$assistantConte2 : undefined;
      const kbDataClient = (_await$assistantConte3 = await assistantContext.getAIAssistantKnowledgeBaseDataClient()) !== null && _await$assistantConte3 !== void 0 ? _await$assistantConte3 : undefined;
      const dataClients = {
        anonymizationFieldsDataClient,
        conversationsDataClient,
        kbDataClient
      };

      // esStore
      const esStore = new _elasticsearch_store.ElasticsearchStore(esClient, (_kbDataClient$indexTe = kbDataClient === null || kbDataClient === void 0 ? void 0 : (_kbDataClient$indexTe2 = kbDataClient.indexTemplateAndPattern) === null || _kbDataClient$indexTe2 === void 0 ? void 0 : _kbDataClient$indexTe2.alias) !== null && _kbDataClient$indexTe !== void 0 ? _kbDataClient$indexTe : '', logger, telemetry, elserId, _constants.ESQL_RESOURCE, kbDataClient);

      // Actions
      const actionsClient = await actions.getActionsClientWithRequest(request);
      const connectors = await actionsClient.getBulk({
        ids: connectorIds,
        throwIfSystemAction: false
      });

      // Fetch any tools registered to the security assistant
      const assistantTools = assistantContext.getRegisteredTools(_helpers.DEFAULT_PLUGIN_NAME);
      const graphs = await Promise.all(connectors.map(async connector => {
        var _dataClients$anonymiz, _await$dataClients$kb, _dataClients$kbDataCl;
        const llmType = (0, _utils2.getLlmType)(connector.actionTypeId);
        const isOpenAI = llmType === 'openai';
        const llmClass = (0, _utils2.getLlmClass)(llmType, true);
        const createLlmInstance = () => new llmClass({
          actionsClient,
          connectorId: connector.id,
          llmType,
          logger,
          temperature: (0, _server2.getDefaultArguments)(llmType).temperature,
          signal: abortSignal,
          streaming: false,
          maxRetries: 0
        });
        const llm = createLlmInstance();
        const anonymizationFieldsRes = await (dataClients === null || dataClients === void 0 ? void 0 : (_dataClients$anonymiz = dataClients.anonymizationFieldsDataClient) === null || _dataClients$anonymiz === void 0 ? void 0 : _dataClients$anonymiz.findDocuments({
          perPage: 1000,
          page: 1
        }));
        const anonymizationFields = anonymizationFieldsRes ? (0, _helpers2.transformESSearchToAnonymizationFields)(anonymizationFieldsRes.data) : undefined;
        const modelExists = await esStore.isModelInstalled();

        // Create a chain that uses the ELSER backed ElasticsearchStore, override k=10 for esql query generation for now
        const chain = _chains.RetrievalQAChain.fromLLM(llm, esStore.asRetriever(10));

        // Check if KB is available
        const isEnabledKnowledgeBase = (_await$dataClients$kb = await ((_dataClients$kbDataCl = dataClients.kbDataClient) === null || _dataClients$kbDataCl === void 0 ? void 0 : _dataClients$kbDataCl.isModelDeployed())) !== null && _await$dataClients$kb !== void 0 ? _await$dataClients$kb : false;

        // Skeleton request from route to pass to the agents
        // params will be passed to the actions executor
        const skeletonRequest = {
          ...request,
          body: {
            alertsIndexPattern: '',
            allow: [],
            allowReplacement: [],
            subAction: 'invokeAI',
            // The actionTypeId is irrelevant when used with the invokeAI subaction
            actionTypeId: '.gen-ai',
            replacements: {},
            size: DEFAULT_SIZE,
            conversationId: ''
          }
        };

        // Fetch any applicable tools that the source plugin may have registered
        const assistantToolParams = {
          anonymizationFields,
          chain,
          esClient,
          isEnabledKnowledgeBase,
          kbDataClient: dataClients === null || dataClients === void 0 ? void 0 : dataClients.kbDataClient,
          llm,
          logger,
          modelExists,
          request: skeletonRequest,
          alertsIndexPattern,
          // onNewReplacements,
          replacements,
          size
        };
        const tools = assistantTools.flatMap(tool => {
          var _tool$getTool;
          return (_tool$getTool = tool.getTool(assistantToolParams)) !== null && _tool$getTool !== void 0 ? _tool$getTool : [];
        });
        const agentRunnable = isOpenAI ? await (0, _agents.createOpenAIFunctionsAgent)({
          llm,
          tools,
          prompt: _prompts.openAIFunctionAgentPrompt,
          streamRunnable: false
        }) : llmType && ['bedrock', 'gemini'].includes(llmType) ? (0, _agents.createToolCallingAgent)({
          llm,
          tools,
          prompt: llmType === 'bedrock' ? _prompts.bedrockToolCallingAgentPrompt : _prompts.geminiToolCallingAgentPrompt,
          streamRunnable: false
        }) : await (0, _agents.createStructuredChatAgent)({
          llm,
          tools,
          prompt: _prompts.structuredChatAgentPrompt,
          streamRunnable: false
        });
        return {
          name: `${runName} - ${connector.name}`,
          llmType,
          graph: (0, _graph.getDefaultAssistantGraph)({
            agentRunnable,
            dataClients,
            createLlmInstance,
            logger,
            tools,
            replacements: {}
          })
        };
      }));

      // Run an evaluation for each graph so they show up separately (resulting in each dataset run grouped by connector)
      await (0, _std.asyncForEach)(graphs, async ({
        name,
        graph,
        llmType
      }) => {
        // Wrapper function for invoking the graph (to parse different input/output formats)
        const predict = async input => {
          logger.debug(`input:\n ${JSON.stringify(input, null, 2)}`);
          const r = await graph.invoke({
            input: input.input,
            conversationId: undefined,
            responseLanguage: 'English',
            llmType,
            bedrockChatEnabled: true,
            isStreaming: false
          },
          // TODO: Update to use the correct input format per dataset type
          {
            runName,
            tags: ['evaluation']
          });
          const output = r.agentOutcome.returnValues.output;
          return output;
        };
        const evalOutput = await (0, _evaluation.evaluate)(predict, {
          data: datasetName !== null && datasetName !== void 0 ? datasetName : '',
          evaluators: [],
          // Evals to be managed in LangSmith for now
          experimentPrefix: name,
          client: new _langsmith.Client({
            apiKey: langSmithApiKey
          })
        });
        logger.debug(`runResp:\n ${JSON.stringify(evalOutput, null, 2)}`);
      });
      return response.ok({
        body: {
          evaluationId,
          success: true
        }
      });
    } catch (err) {
      logger.error(err);
      const error = (0, _securitysolutionEsUtils.transformError)(err);
      const resp = (0, _build_response.buildResponse)(response);
      return resp.error({
        body: {
          success: false,
          error: error.message
        },
        statusCode: error.statusCode
      });
    }
  });
};
exports.postEvaluateRoute = postEvaluateRoute;